"use server";

import Docker from 'dockerode';

const docker = new Docker();
const CONTAINER_TIMEOUT = 60;     // 容器操作超时时间（秒）
const PREWARM_COUNT = 2;          // 预热容器数量配置
const SQL_SERVER_STARTUP_DELAY = 15000; // SQL Server启动等待时间（毫秒）

// 端口管理配置
const PORT_CONFIG = {
  min: 5000,
  max: 6000,
  current: 5000
};

// 容器池数据结构
type ContainerPool = {
  [key: string]: {
    free: Docker.ContainerInfo[];
    used: Map<string, Docker.ContainerInfo>;
  }
};

const containerPool: ContainerPool = {
  sqlserver: { free: [], used: new Map() },
  mysql: { free: [], used: new Map() }
};

// 初始化预热容器
(async () => {
  await Promise.all([
    ...Array(PREWARM_COUNT).fill(null).map(() => createContainer('sqlserver')),
    ...Array(PREWARM_COUNT).fill(null).map(() => createContainer('mysql'))
  ]);
  console.log('预热容器初始化完成');
})();

// 安全容器操作方法
async function safeStopContainer(container: Docker.Container) {
  try {
    await container.stop();
  } catch (error: any) {
    if (![404, 409].includes(error.statusCode)) {
      console.error('停止容器失败:', error);
      throw error;
    }
  }
}

async function safeRemoveContainer(container: Docker.Container) {
  try {
    await container.remove();
  } catch (error: any) {
    if (error.statusCode !== 404) {
      console.error('删除容器失败:', error);
      throw error;
    }
  }
}

// 容器创建逻辑
async function createContainer(dbType: string) {
  const port = getAvailablePort();
  const config = getDatabaseConfig(dbType);

  try {
    const container = await docker.createContainer({
      Image: config.image,
      Env: config.env,
      HostConfig: {
        AutoRemove: true,
        PortBindings: {
          [dbType === 'sqlserver' ? '1433/tcp' : '3306/tcp']: [
            { HostPort: port.toString() }
          ]
        },
        ...(dbType === 'sqlserver' && {
          Memory: 2147483648,
          NetworkMode: 'bridge'
        })
      }
    });

    await container.start();

    // SQL Server需要额外启动等待时间
    if (dbType === 'sqlserver') {
      await new Promise(resolve => setTimeout(resolve, SQL_SERVER_STARTUP_DELAY));
    }

    // 执行健康检查
    if (await healthCheck(container, dbType)) {
      const info = await container.inspect();
      containerPool[dbType].free.push({
        ...info,
        Port: port
      });
      console.log(`已创建 ${dbType} 容器：${info.Id}`);
    }
  } catch (error) {
    console.error(`创建 ${dbType} 容器失败:`, error);
    throw error;
  }
}

// 健康检查逻辑（增强版）
async function healthCheck(container: Docker.Container, dbType: string) {
  const config = getDatabaseConfig(dbType);
  const containerInfo = await container.inspect();
  const containerIp = containerInfo.NetworkSettings.Networks.bridge?.IPAddress;

  if (!containerIp) {
    console.error('无法获取容器IP地址');
    await safeRemoveContainer(container);
    return false;
  }

  const testCommand = dbType === 'sqlserver'
    ? `/opt/mssql-tools18/bin/sqlcmd -S ${containerIp},1433 -U SA -P ${config.password} -Q "SELECT 1" -C -N -l 30`
    : `mysql -h ${containerIp} -u root -p${config.password} -e "SELECT 1"`;

  try {
    const exec = await container.exec({
      Cmd: ['sh', '-c', testCommand],
      AttachStdout: true,
      AttachStderr: true
    });

    // 设置30秒超时
    const timeoutPromise = new Promise((_, reject) =>
      setTimeout(() => reject(new Error('健康检查超时')), 30000)
    );

    await Promise.race([
      new Promise((resolve, reject) => {
        exec.start((err, stream) => {
          if (err) return reject(err);
          stream?.on('end', resolve);
          stream?.on('error', reject);
          stream?.resume();
        });
      }),
      timeoutPromise
    ]);

    return true;
  } catch (error) {
    console.error(`健康检查失败: ${error}`);
    await safeRemoveContainer(container);
    return false;
  }
}

// 数据库配置（增加SSL参数）
function getDatabaseConfig(dbType: string) {
  const password = 'YourStrong!Passw0rd';
  return {
    password,
    image: dbType === 'sqlserver'
      ? 'mcr.microsoft.com/mssql/server:2019-latest'
      : 'mysql:latest',
    env: dbType === 'sqlserver' ? [
      'ACCEPT_EULA=Y',
      `SA_PASSWORD=${password}`,
      'MSSQL_AGENT_ENABLED=True',
      'MSSQL_TCP_PORT=1433',
      'MSSQL_LCID=1033',
      'MSSQL_MEMORY_LIMIT_MB=2048',
      'MSSQL_OPTION_FLAGS=-C' // 启用信任服务器证书
    ] : [
      `MYSQL_ROOT_PASSWORD=${password}`,
      'MYSQL_TCP_PORT=3306',
      'MYSQL_ROOT_HOST=%'
    ]
  };
}

// 端口分配
function getAvailablePort() {
  PORT_CONFIG.current =
    PORT_CONFIG.current >= PORT_CONFIG.max
      ? PORT_CONFIG.min
      : PORT_CONFIG.current + 1;
  return PORT_CONFIG.current;
}

// 主服务函数（关键修复点）
export default async function serverAction(
  sql: string,
  dbType: string
): Promise<string> {
  let containerInfo: Docker.ContainerInfo | undefined;

  try {
    // 获取或创建容器
    if (containerPool[dbType].free.length === 0) {
      await createContainer(dbType);
    }

    containerInfo = containerPool[dbType].free.pop()!;
    containerPool[dbType].used.set(containerInfo.Id, containerInfo);

    const container = docker.getContainer(containerInfo.Id);
    const containerIp = containerInfo.NetworkSettings.Networks.bridge?.IPAddress;

    if (!containerIp) {
      throw new Error('无法获取容器IP地址');
    }

    // 构建执行命令（增加SSL参数）
    const command = dbType === 'sqlserver'
      ? `/opt/mssql-tools18/bin/sqlcmd -S ${containerIp},1433 -U SA -P ${getDatabaseConfig(dbType).password} -Q "${sql}" -C -N -l 30`
      : `mysql -h ${containerIp} -u root -p${getDatabaseConfig(dbType).password} -e "${sql}"`;

    // 执行SQL命令
    const exec = await container.exec({
      Cmd: ['sh', '-c', command],
      AttachStdout: true,
      AttachStderr: true
    });

    // 增加执行状态跟踪
    let isCompleted = false;
    const cleanup = async () => {
      if (!isCompleted && containerInfo) {
        console.log('执行未完成，执行清理...');
        await safeRemoveContainer(container);
        containerPool[dbType].used.delete(containerInfo.Id);
        await createContainer(dbType);
      }
    };

    const result = await new Promise<string>((resolve, reject) => {
      let output = '';
      exec.start({}, async (err, stream) => {
        if (err) {
          await cleanup();
          return reject(err);
        }

        stream?.on('data', (chunk: Buffer) => {
          output += chunk.toString();
        });

        stream?.on('end', async () => {
          try {
            if (dbType === 'mysql') {
              output = output.replace(/Warning: Using a password on the command line interface can be insecure.\n/g, '');
            }
            isCompleted = true;

            // 安全归还容器
            await safeStopContainer(container);
            containerPool[dbType].free.push(containerInfo!);
            containerPool[dbType].used.delete(containerInfo!.Id);

            resolve(output);
          } catch (cleanupError) {
            await cleanup();
            reject(cleanupError);
          }
        });

        stream?.on('error', async (error) => {
          await cleanup();
          reject(error);
        });

        stream?.resume();
      });
    });

    return result;
  } catch (error) {
    if (containerInfo) {
      await safeRemoveContainer(docker.getContainer(containerInfo.Id));
      containerPool[dbType].used.delete(containerInfo.Id);
      await createContainer(dbType);
    }
    console.error('执行错误:', error);
    throw error instanceof Error ? error : new Error('未知错误');
  }
}

// 后台维护任务
setInterval(async () => {
  try {
    for (const dbType of ['sqlserver', 'mysql']) {
      // 维持预热容器数量
      while (containerPool[dbType].free.length < PREWARM_COUNT) {
        await createContainer(dbType);
      }

      // 清理无效容器
      containerPool[dbType].free = (await Promise.all(
        containerPool[dbType].free.map(async (info) => {
          try {
            const container = docker.getContainer(info.Id);
            await container.inspect();
            return info;
          } catch {
            return null;
          }
        })
      )).filter(Boolean) as Docker.ContainerInfo[];
    }
  } catch (error) {
    console.error('维护任务出错:', error);
  }
}, 30000); // 每30秒执行一次维护
